/**
 * Convenience functions for cluster.
 * @memberof module:apeman/util/clustering
 * @constructor ClusterHelper
 * @param {string} basedir - Project base directory.
 * @author Taka Okunishi
 */

"use strict";

var u = require('../../lib'),
    childProcess = u.core.childProcess,
    async = u.ext.async,
    file = u.file,
    os = u.core.os,
    fs = u.core.fs,
    path = u.core.path,
    json = u.json;


function ClusterHelper() {
    var s = this;
    s.init.apply(s, arguments);
    Object.defineProperties(s, {
        async: {
            get: function () {
                return async;
            }
        },
        persist: {
            get: function () {
                return require('./persist');
            }
        }
    });
}

/**
 * Initialize the helper.
 * @param {string} basedir - Project base directory.
 */
ClusterHelper.prototype.init = function (basedir) {
    var s = this;
    s.basedir = basedir || process.cwd();
};


/**
 * Get number of CPUs.
 * @returns {*}
 */
ClusterHelper.prototype.numberOfCPUs = function () {
    return os.cpus().length;
};

/**
 * Kill a process.
 * @param {string[]} args - Kill command arguments.
 * @param {string} pid - Process id.
 * @param {function} callback - Callback when done.
 */
ClusterHelper.prototype.kill = function (args, pid, callback) {
    var kill = childProcess.spawn('kill', [].concat(args || []).concat(pid));
    kill.stdout.pipe(process.stdout);
    kill.stderr.pipe(process.stderr);
    kill.on('close', function (code) {
        var success = code === 0,
            err = success ? null : new Error('Kill command exist with code: ' + code);
        callback(err);
    });
};

/**
 * Status file name.
 * @returns {string} - Cluster filename.
 */
ClusterHelper.prototype.statusFilename = function () {
    var s = this;
    return path.resolve(s.basedir, '.cluster_status.json');
};

/**
 * Write the status file.
 * @param {object} data - Data to write.
 * @param {function} [callback] - Callback when done.
 */
ClusterHelper.prototype.writeStatus = function (data, callback) {
    var s = this,
        filename = s.statusFilename();
    json.writeJsonFile(filename, data, callback);
};

/**
 * Read status data.
 * @param {function} callback - Callback when done.
 */
ClusterHelper.prototype.readStatus = function (callback) {
    var s = this,
        filename = s.statusFilename();
    fs.exists(filename, function (exists) {
        if (!exists) {
            callback(null, {});
            return;
        }
        json.readJsonFile(filename, callback);
    });
};


/**
 * Fork a task.
 * @param {string} bin - Bin file to fork.
 * @param {string[]} args - Arguments.
 * @param {function} callback - Callback when done.
 */
ClusterHelper.prototype.fork = function (bin, args, callback) {
    var forked = childProcess.fork(bin, args, {
        silent: false,
        env: process.env
    });
    forked.on('error', function (err) {
        if (callback) {
            callback(err);
            callback = null;
        }
    });
    forked.on('exit', function () {
        if (callback) {
            callback(null);
        }
    });
};

/**
 * Fork workers
 * @param {object} cluster - Node cluster
 * @param {number} numberOfWorkers - Number of workders to fork.
 * @param {function} callback - Callback when done.
 */
ClusterHelper.prototype.forkClusterWorkers = function forkWorkers(cluster, numberOfWorkers, callback) {
    for (var i = 0; i < numberOfWorkers; i++) {
        cluster.fork();
    }
    var listeningCount = 0;
    var workers = [];
    cluster.on('listening', function (worker, address) {
        workers.push(worker);
        listeningCount++;
        var ready = listeningCount == numberOfWorkers;
        if (ready) {
            if (callback) {
                callback(workers);
            }
        }
    });
}

/**
 * Bind cluster worker
 * @param {object} cluster
 * @param {boolean} shouldRefork - Should re-fork or not.
 */
ClusterHelper.prototype.bindReforkClusterWorker = function (cluster, shouldRefork) {
    cluster.on('disconnect', function (worker) {
        var refork = shouldRefork(worker);
        if (refork) {
            cluster.fork();
        }
    });
}

/**
 * Send data to all workers.
 * @param {object} cluster
 * @param {object} data - Data to send.
 */
ClusterHelper.prototype.sendToAllClusterWorkers = function (cluster, data) {
    for (var id in cluster.workers) {
        if (cluster.workers.hasOwnProperty(id)) {
            var worker = cluster.workers[id];
            worker.send(data);
        }
    }
};

/**
 * Handle async done callback.
 * @param {string} msg - Message to console.
 * @returns {function} done handler.
 */
ClusterHelper.prototype.done = function (msg) {
    return function (err) {
        if (err) {
            console.error(err)
        } else {
            console.log(msg);
        }
        process.exit(err ? 1 : 0);
    }
};


module.exports = ClusterHelper;