/**
 * Form validator
 * @memberof module:apeman-util/lib/form
 * @constructor FormValidator
 * @params {object} config - validator configuration.
 * @see https://github.com/acornejo/jjv
 * @author Taka Okunishi
 */

"use strict";


var ext = require('../ext'),
    core = require('../core'),
    copy = require('../object/copy'),
    format = core.util.format,
    jjv = ext.jjv;

/**
 * @lends FormValidator
 * @constructor
 */
function FormValidator(config) {
    var s = this;
    s.init.apply(s, arguments);
}

FormValidator.prototype = {
    init: function (config) {
        var s = this;
        copy(config || {}, s);
        s.env = s.env || jjv();
        s.locales = s.locales || {};
    },
    lang: 'en',
    locales: null,
    env: null,
    /**
     * Add schema to the validator.
     * @param {string} name - Schema name.
     * @param {object} schema - Json schema object.
     */
    addSchema: function (name, schema) {
        var s = this;
        if (typeof(arguments[0]) === 'object') {
            var schemas = arguments[0];
            Object.keys(schemas).forEach(function (name) {
                var schema = schemas[name];
                s.addSchema(name, schema);
            });
            return s;
        }
        s.env.addSchema(name, schema);
        return s;
    },
    addLocale: function (lang, locale) {
        var s = this;
        s.locales[lang] = locale;
        return s;
    },
    getLocale: function () {
        var s = this;
        return s.locales[s.lang] || {};
    },
    setLang: function (lang) {
        var s = this;
        s.lang = lang;
    },
    /**
     * Do validate.
     * @param {string} name - Schema name.
     * @param {object} values - Values to validate.
     */
    validate: function (name, values) {
        values = values || {};
        var s = this,
            schema = s.env.schema[name];
        if (!schema) {
            throw new Error('[FormValidator]No schema found with name: ' + name);
        }
        var err = s.env.validate(name, values);
        if (!err) {
            return null;
        }
        var result = {};
        Object.keys(err.validation || {}).forEach(function (property) {
            var reasons = err.validation[property];
            result[property] = {
                reason: reasons,
                message: s.message(property, schema.properties[property], reasons)
            };
        });
        return result;
    },
    /**
     * Create message for an error.
     * @param {string} property - Property name working with.
     * @param {string} def - Schema def.
     * @param {object} reasons - Error reason.
     */
    message: function (property, def, reasons) {
        var s = this,
            locale = s.getLocale(),
            propertyLabel = (locale.properties || locale.property || locale)[property],
            data = copy(def || {}, {property: propertyLabel || property});
        return Object.keys(reasons)
            .map(function (key) {
                var val = reasons[key];
                var msg = (locale.errors || locale.error || locale)[key] || s._defaultMessage(key, def[key] || val);
                return s._format(msg, data);
            });
    },
    _defaultMessage: function (key, val) {
        var reason = {};
        reason[key] = val;
        return format('"{property}" does not to confirm schema: %s', JSON.stringify(reason));
    },
    _format: function (msg, data) {
        var s = this;
        Object.keys(data).forEach(function (key) {
            var val = data[key];
            if (Array.isArray(val)) {
                msg = msg.replace('{' + key + '}', val.join(','));
                val.forEach(function (val, i) {
                    msg = msg.replace('{' + key + '[' + i + ']}', val);
                });
            } else if (typeof(val) === 'object') {
                msg = s._format(msg, val);
            } else {
                msg = msg.replace('{' + key + '}', val);
            }
        });
        return msg;
    }

};

module.exports = FormValidator;
