/**
 * Covert to inlined html.
 * @memberof moudle:apeman-util/lib/html
 * @function inlineHtml
 * @param {string} src - Src file name.
 * @apram {string} dest - Destination file name.
 * @param {string} callback - Callbakck when done.
 */

"use strict";

var u = require('../../lib'),
    async = u.ext.async,
    request = u.core.request,
    path = u.core.path,
    imaging = u.imaging,
    fs = u.core.fs;

/** @lends inlineHtml */
function inlineHtml(src, dest, callback) {
    src = path.resolve(src);
    dest = path.resolve(dest);
    async.waterfall([
        function (callback) {
            inlineHtml.load(src, callback);
        },
        function ($, callback) {
            inlineHtml.inline(path.dirname(src), $, callback);
        },
        function (html, callback) {
            fs.writeFile(dest, html, callback);
        }
    ], callback);
}


inlineHtml.isRemote = function (pathname) {
    return pathname && pathname.match(/^http/);
};

inlineHtml.read = function (filename, callback) {
    var isRemote = inlineHtml.isRemote(filename);
    if (isRemote) {
        request(filename, function (err, response, body) {
            callback(err, body);
        });
    } else {
        fs.readFile(filename, callback);
    }
}

inlineHtml.toBase64 = function (filename, callback) {
    async.waterfall([
        function (callback) {
            inlineHtml.read(filename, callback);
        },
        function (buffer, callback) {
            var extname = path.extname(filename);
            var base64String = imaging.base64String(buffer, extname.replace(/^\./, ''));
            callback(null, base64String);
        }
    ], callback);
}

inlineHtml.inline = function (dirname, $, callback) {
    function resolve(filename) {
        var isRemote = inlineHtml.isRemote(filename);
        return isRemote ? filename : path.resolve(dirname, filename);
    }


    function find(selector, options) {
        var required = options && options.required;
        return $(selector).toArray()
            .map(function (elm) {
                return $(elm);
            })
            .filter(function ($elm) {
                if (required) {
                    return !!$elm.attr(required);
                }
                return true;
            });
    }

    function replace(from, to, callback) {
        from.after(to);
        from.remove();
        callback(null);
    }

    async.series([
        function convertStyles(callback) {
            var $links = find('link[rel="stylesheet"]', {required: 'href'});
            async.eachSeries($links, function ($link, callback) {
                var href = resolve($link.attr('href'));
                async.waterfall([
                    function (callback) {
                        inlineHtml.inline.toStyleTag(href, callback);
                    },
                    function (style, callback) {
                        replace($link, style, callback);
                    }
                ], callback);
            }, callback);
        },
        function convertScripts(callback) {
            var $scripts = find('script', {required: 'src'});
            async.eachSeries($scripts, function ($script, callback) {
                var src = resolve($script.attr('src'));
                async.waterfall([
                    function (callback) {
                        inlineHtml.inline.toScriptTag(src, callback);
                    },
                    function (script, callback) {
                        replace($script, script, callback);
                    }
                ], callback);
            }, callback);
        },
        function convertFavicons(callback) {
            var $links = find('link[rel="shortcut icon"],link[rel="icon"]', {required: 'href'});
            async.eachSeries($links, function ($link, callback) {
                var href = resolve($link.attr('href'));
                async.waterfall([
                    function (callback) {
                        inlineHtml.toBase64(href, callback);
                    },
                    function (href, callback) {
                        $link.attr('href', href);
                        callback(null);
                    }
                ], callback);
            }, callback);
        },
        function convertImages(callback) {
            var $imgs = find('img', {required: 'src'});
            async.eachSeries($imgs, function ($img, callback) {
                var src = resolve($img.attr('src'));
                async.waterfall([
                    function (callback) {
                        inlineHtml.toBase64(src, callback);
                    },
                    function (src, callback) {
                        $img.attr('src', src);
                        callback(null);
                    }
                ], callback);
            }, callback);
        }
    ], function (err) {
        callback(err, $.html());
    });
}


inlineHtml.inline.toTag = function (filename, tags, callback) {
    if (!filename) {
        callback(null, '');
        return;
    }
    async.waterfall([
        function (callback) {
            inlineHtml.read(filename, callback);
        },
        function (data, callback) {
            var string = [
                tags[0],
                    '\t' + String(data).replace(/\n/g, "\n\t\t"),
                tags[1]
            ].join('\n\t');
            callback(null, string);
        }
    ], callback);
}

inlineHtml.inline.toStyleTag = function (filename, callback) {
    inlineHtml.inline.toTag(filename, [
        '<style type="text/css">',
        '</style>'
    ], callback);
}

inlineHtml.inline.toScriptTag = function (filename, callback) {
    inlineHtml.inline.toTag(filename, [
        '<script type="text/javascript">',
        '</script>'
    ], callback);
}


inlineHtml.load = function (src, callback) {
    var cheerio;
    try {
        cheerio = require('cheerio');
    } catch (e) {
        var error = new Error('cheerio not found.Try `npm install cheerio --save`');
        callback(error);
        return;
    }
    async.waterfall([
        function (callback) {
            inlineHtml.read(src, callback);
        },
        function (buffer, callback) {
            var $ = cheerio.load(String(buffer), {
                normalizeWhitespace: false,
                decodeEntities: false,
                xmlMode: false
            });
            callback(null, $);
        }
    ], callback);
}

module.exports = inlineHtml;