/**
 * Json schema.
 * @memberof module:apeman-util/lib/schemas
 * @augments Schema
 * @constructor JsonSchema
 * @param {object} def - Schema definition.
 * @see {@link http://json-schema.org/}
 * @see {@link module:apeman-util|util}
 * @author Taka Okunishi
 */

"use strict";

var Schema = require('./../schema'),
    definitions = require('./definitions'),
    copy = require('../../object/copy'),
    ext = require('../../ext'),
    tv4 = ext.tv4;

/** @lends JsonSchema */
function JsonSchema() {
    var s = this;
    s.init.apply(s, arguments);
}

copy(Schema, JsonSchema);
JsonSchema.prototype = new Schema({});

/**
 * Initialize schema.
 * @param {object} def - Schema definition.
 * @param {string} [lang="en"] - lang
 */
JsonSchema.prototype.init = function (def, lang) {
    var s = this,
        isJsonSchema = def instanceof(JsonSchema);
    s.def = isJsonSchema ? def.def : def;
    s.tv4 = tv4.freshApi();

    var langData = JsonSchema.defulatLangData;
    Object.keys(langData).forEach(function (lang) {
        s.addLangData(lang, langData[lang]);
    });
    s.setLang(lang || 'en');
};


/**
 * Set language.
 * @param {string} lang - Lang;
 */
JsonSchema.prototype.setLang = function (lang) {
    var s = this;
    s.tv4.language(lang);
};

/**
 * Add lang data.
 * @param {string} lang - Lang.
 * @param {object} data - Lang data.
 */
JsonSchema.prototype.addLangData = function (lang, data) {
    var s = this;
    s.tv4.addLanguage(lang, data);
};

/**
 * tv4 instance.
 * @type {null}
 */
JsonSchema.prototype.tv4 = null;

/**
 * Validate values.
 * @param {object} values - Object to validate.
 * @param {function} callback - Callback when done.
 */
JsonSchema.prototype.validate = function (values, callback) {
    var s = this,
        report = s.tv4.validateMultiple(values, s.def);
    callback(null, report);
};

/**
 * Throws a error unless valid.
 * @param {object} values - Object to validate.
 * @throws {Error} - Not valid error.
 */
JsonSchema.prototype.throwIfError = function (values) {
    var s = this;
    s.validate(values, function (err, report) {
        var valid = report.valid;
        if (!valid) {
            throw new Error(report.errors[0].message);
        }
    });
};

/**
 * Get error name.
 * @param {string} code - Error code.
 */
JsonSchema._errorKind = function (code) {
    for (var key in tv4.errorCodes) {
        if (tv4.errorCodes.hasOwnProperty(key)) {
            var hit = tv4.errorCodes[key] === code;
            if (hit) {
                return key;
            }
        }
    }
    return null;
};

/**
 * Error kinds.
 * @type {*|Array}
 */
JsonSchema.ERROR_KINDS = Object.keys(tv4.errorCodes);

/**
 * Schema definitions.
 * @borrows module:apeman-util/schemas.definitions as definitions
 */
JsonSchema.definitions = definitions;

/**
 * Default lang data.
 * @type {exports}
 */
JsonSchema.defulatLangData = require('../_schema_error_formats');

module.exports = JsonSchema;