/**
 * Helper for task.
 * @memberof module:apeman-util/lib/tasking
 * @constructor TaskHelper
 * @param {string} basedir - Base directory path.
 * @see {@link module:apeman-util|util}
 * @author Taka Okunishi
 */

"use strict";

var u = require('../../lib'),
    childProcess = u.core.childProcess,
    ext = u.ext,
    core = u.core,
    file = u.file,
    yesno = ext.yesno,
    async = ext.async,
    fs = core.fs,
    path = core.path;

/** @lends TaskHelper */
function TaskHelper(basedir) {
    var s = this;
    s.init(basedir || process.cwd());
}

/**
 * Initialize task helper.
 * @param {string} basedir - Base directory path.
 */
TaskHelper.prototype.init = function (basedir) {
    var s = this;
    s.basedir = basedir;
};

/** @borrows async */
TaskHelper.prototype.async = async;
/** @borrows yesno */
TaskHelper.prototype.yesno = yesno;
/** @borrows fs */
TaskHelper.prototype.fs = fs;
/** @borrows path */
TaskHelper.prototype.path = path;


/**
 * Done function.
 * @param {string} msg - Message to display.
 * @param {boolean} [exit=false] - Exit when done.
 * @returns {function} - Done function.
 */
TaskHelper.prototype.done = function (msg, exit) {
    return function (err) {
        if (err) {
            console.error(err);
        } else {
            console.log(msg);
        }
        if (!!exit) {
            process.nextTick(function () {
                var exitCode = err ? 1 : 0;
                process.exit(exitCode);
            });
        }
    }
};

/**
 * Watch files and run task.
 * @param {string[]} filenames - File names to watch.
 * @param {Task} task - Task to run.
 */
TaskHelper.prototype.watchAndRun = function (filenames, task) {
    task._busyToRun = false;
    file.watchAll(filenames, {persistent: true}, function (event, filename) {
        if (task._busyToRun) {
            return;
        }
        task._busyToRun = true;
        task.run(function () {
            task._busyToRun = false;
        });
    });
};

TaskHelper.prototype.spawn = function (bin, args) {
    var s = this;
    bin = path.resolve(s.basedir, bin);
    var spawned = childProcess.spawn(bin, args);
    spawned.stdout.pipe(process.stdout);
    spawned.stderr.pipe(process.stderr);
    return spawned;
};

module.exports = TaskHelper;