/**
 * Helper for test.
 * @memberof module:apeman-util/lib/testing
 * @constructor TestHelper
 * @param {object} config - Configuration.
 * @param {string} mockDir - Mock files directory.
 * @param {string} workDir - Work files directory.
 * @see {@link module:apeman-util|util}
 * @author Taka Okunishi
 */

"use strict";

var u = require('../../lib'),
    ext = u.ext,
    core = u.core,
    testing = u.testing,
    mkdirp = ext.mkdirp,
    async = ext.async,
    fs = core.fs,
    assert = core.assert,
    path = core.path;


/** @lends TestHelper */
function TestHelper(config) {
    var s = this;
    s.init(config || {});
}

/**
 * Initialize the test helper.
 * @param {object} config - Init configuration.
 */
TestHelper.prototype.init = function (config) {
    var s = this;
    s.mockDir = config.mockDir || 'test/mock';
    s.workDir = config.workDir || 'test/.work';
    s.injector = config.injector || new testing.MockInjector();
};

/**
 * Do nothing.
 */
TestHelper.prototype.doNothing = function () {
};

/**
 * Resolve a mock file path.
 * @param {string} filename - Mock filename.
 * @returns {string} - Resolved file name.
 */
TestHelper.prototype.resolveMockFile = function (filename) {
    var s = this;
    return path.resolve(s.mockDir, path.join.apply(path, arguments));
};

/**
 * Resolve a work file path.
 * @param {string} filename - Work filename.
 * @returns {string} - Resolved file name.
 */
TestHelper.prototype.resolveWorkFile = function (filename) {
    var s = this;
    return path.resolve(s.workDir, path.join.apply(path, arguments));
};

/**
 * An injector.
 * @type {MockInjector|exports}
 */
TestHelper.prototype.injector = null;


/**
 * ExistsSync
 * @borrows fs.existsSync
 */
TestHelper.prototype.existsSync = fs.existsSync;

/**
 * Prepare clean work directory.
 * @param {string} dirname - Directory name to prepare.
 * @returns {string} - Work directory name.
 */
TestHelper.prototype.prepareCleanWorkDir = function (dirname) {
    var s = this;
    var isExistingFile = fs.existsSync(dirname) && !fs.statSync(dirname).isDirectory();
    if (isExistingFile) {
        throw new Error('A file exists at a path:' + dirname);
    }
    s._rmdirRecursiveSync(dirname);
    mkdirp.sync(dirname);
    return dirname;
};

/**
 * Prepare clean work directory.
 * @param {string} testFile - Test file name.
 * @returns {string} - Work directory name.
 */
TestHelper.prototype.prepareCleanWorkDirForTestFile = function (testFile) {
    var s = this,
        dirname = path.basename(path.dirname(testFile)),
        basename = path.basename(testFile, path.extname(testFile));
    var name = path.join(dirname, basename).split(path.sep).join('_');
    var workDirname = s.resolveWorkFile(name);
    return s.prepareCleanWorkDir(workDirname);
};


/**
 * Remove directory recursively.
 * @param {string} dirname - Directory name.
 * @protected
 * @ignore
 */
TestHelper.prototype._rmdirRecursiveSync = function (dirname) {
    var s = this;
    if (!fs.existsSync(dirname)) return;
    fs.readdirSync(dirname)
        .map(function (filename) {
            return path.resolve(dirname, filename);
        })
        .forEach(function (filename) {
            var isSymlink = fs.lstatSync(filename).isSymbolicLink();
            if (isSymlink) {
                fs.unlinkSync(filename);
            } else {
                var isDir = fs.statSync(filename).isDirectory();
                if (isDir) {
                    s._rmdirRecursiveSync(filename);
                } else {
                    fs.unlinkSync(filename);
                }
            }
        });
    fs.rmdirSync(dirname);
};

TestHelper.prototype.fs = fs;
TestHelper.prototype.path = path;
TestHelper.prototype.test = assert;
TestHelper.prototype.async = async;
TestHelper.prototype.mkdirp = mkdirp;

module.exports = TestHelper;